             .title        "Jaleco/Konami/Taito LS139+LS74 (INES 87) Plugin"

;05/05/07
;BootGod

             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
curbank:    .equ 00e2h
banksleft:    .equ 00e3h
bank0crc:  .equ 00e4h

upaddress:       .equ 00e8h
upaddress_lo:    .equ 00e8h
upaddress_hi:    .equ 00e9h

numpages_lo: .equ 00eah
numpages_hi: .equ 00ebh
tx: .equ 00ebh


temp_crc:    .equ 00ech

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Jaleco/Konami/Taito"
             .db " 139/74 Carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 16 >> 4,32 >> 4
             .db 16 >> 3,32 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             
;determine H or V mirroring

             ldx #00h
             
             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne horz_mir
             lda 2007h
             cmp #0aah
             bne horz_mir
             lda #020h
             jsr wr_ppu
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #0aah
             bne horz_mir
             inx

horz_mir:    txa
             jsr send_byte   ;send byte

;determine if this is a 16K or 32K game
             lda #000h
             sta address_lo
             sta upaddress_lo
             lda #080h
             sta address_hi
             lda #0c0h
             sta upaddress_hi
             
             ldx #040h
             ldy #0

chk_loop:    lda (address),y
             cmp (upaddress),y
             bne not_16k
             iny
             bne chk_loop
             inc address_hi
             inc upaddress_hi
             dex
             bne chk_loop

;this is a 16K game

             lda #64
             sta numpages_lo
             bne got_prg

not_16k:     lda #128
             sta numpages_lo

;send PRG size info

got_prg:     lda numpages_lo		;send PRG header
             jsr send_byte
             lda #0
             jsr send_byte
             lda #001h
             jsr send_byte

             lda #0
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             ldy #0
             ldx numpages_lo		;setup bank size for dump

dump_prg:    lda (address),y
             jsr send_byte
             iny
             bne dump_prg
             inc address_hi
             dex
             bne dump_prg
             

;determine CHR ROM size


;get crc of CHR bank 0
             lda #0
             jsr wr_cbank	;set to CHR bank 0
             jsr crc_cbank

             ldx #3
ld_ccrc:     lda temp_crc,x	;save for comparision to CHR bank 1
             sta bank0crc,x
             dex
             bpl ld_ccrc

;get crc of CHR bank 1
             lda #1
             jsr wr_cbank	;set to CHR bank 1
             jsr crc_cbank
             
;compare crc's of bank 0 and 1
             ldx #3
chk_ccrc:    lda bank0crc,x
             cmp temp_crc,x
             bne crc_cfail
             dex
             bpl chk_ccrc

             lda #64		;crc matches, so CHR ROM is 16K
             sta numpages_lo
             lda #2
             sta banksleft
             jmp got_chr

crc_cfail:   lda #128		;crc didn't match so CHR ROM is 32K
             sta numpages_lo	
             lda #4
             sta banksleft
             
got_chr:     
             lda #0
             sta curbank

             lda numpages_lo	;send CHR header
             jsr send_byte
             lda #0
             jsr send_byte
             lda #002h
             jsr send_byte
             
next_chr:    lda curbank
             jsr wr_cbank

             inc curbank
             inc curbank
             lda #4
             cmp curbank
             bne chr_cont
             lda #1
             sta curbank

chr_cont:    lda #0
             jsr wr_ppu
             lda 2007h

             ldx #32
             ldy #0

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             dec banksleft
             bne next_chr

finished:    lda #000h		;send termination header
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte

             rts

wr_cbank:    sta 6000h
             rts

;CRC routine for 8K CHR bank @ $0000, stores crc in temp_crc

crc_cbank:   lda #0
             jsr wr_ppu
             lda 2007h		;set PPU address to $0000

             jsr init_crc
             ldx #32
             ldy #0		;init crc for 8K of data (32 pages = 8K)

crc_cloop:   lda 2007h		;crc generation loop
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne crc_cloop
             dex
             bne crc_cloop
             
             jsr finish_crc	;crc done

             ldx #3
crc_csave:   lda crc0,x		;save crc in temp_crc
             sta temp_crc,x
             dex
             bpl crc_csave

             rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
